# -*- coding: utf-8 -*-
"""
Created on Mon Mar 16 07:44:16 2020

@author: Marie-Laure
"""

import numpy as np
import random
import pygame
LARGEUR_ECRAN = 640 
HAUTEUR_ECRAN = 480
NB_LIGNE = 8
NB_COLONNE=8
JOUEUR=1
ORDINATEUR=2
VISU = 3
CARRE = 50
CADRE = 6
MARGEY = int((HAUTEUR_ECRAN - (NB_LIGNE * CARRE)) / 2 - CADRE)
MARGEX = int((LARGEUR_ECRAN - (NB_COLONNE * CARRE)) / 2 -CADRE)

#              R    G    B
BLANC      = (255, 255, 255)
NOIR      = (  0,   0,   0)
VERT      = (  0, 155,   0)
BLEUCLAIR = (  0,  50, 255)
BRUN      = (174,  94,   0)
COULEURLIGNE = NOIR

# =============================================================================
# Fonctions
# =============================================================================
#---------------Graphique ---------------------------------------------------
#traduit le numéro de ligne et de colonne en coordonnées écran
def traduirePlateauEnCoordEcran(x, y):
    return int(MARGEX+CADRE/2 + (x-1) * CARRE + int(CARRE / 2)), int(MARGEY+CADRE/2 + (y-1) * CARRE + int(CARRE / 2))

#dessine le plateau sur l'écran
def dessiner_plateau(plateau):
    #bouton "Nouveau jeu"
    global zone_NouveauJeu
    surf_NouveauJeu = FONT.render('Nouveau Jeu', True, BLANC, VERT)
    zone_NouveauJeu = surf_NouveauJeu.get_rect()
    zone_NouveauJeu.topright = (LARGEUR_ECRAN - 8, 10)
    ecran.blit(surf_NouveauJeu, zone_NouveauJeu)
    #pourtour du plateau
    pygame.draw.rect(ecran,BRUN,(MARGEX,MARGEY,(NB_COLONNE * CARRE)+CADRE,(NB_LIGNE * CARRE)+CADRE),CADRE)
    #plateau vert
    pygame.draw.rect(ecran,VERT,(MARGEX+CADRE/2,MARGEY+CADRE/2,NB_COLONNE * CARRE,NB_LIGNE * CARRE))
    # Trace les lignes sur le plateau
    for x in range(NB_COLONNE + 1):
        # Lignes horizontales
        startx = (x * CARRE) + MARGEX+CADRE/2
        starty = MARGEY+CADRE/2
        endx = (x * CARRE) + MARGEX+CADRE/2
        endy = MARGEY+CADRE/2 + (NB_COLONNE * CARRE)
        pygame.draw.line(ecran, NOIR, (startx, starty), (endx, endy))
    for y in range(NB_LIGNE + 1):
        # Lignes verticales
        startx = MARGEX+CADRE/2
        starty = (y * CARRE) + MARGEY+CADRE/2
        endx = MARGEX+CADRE/2 + (NB_COLONNE * CARRE)
        endy = (y * CARRE) + MARGEY+CADRE/2
        pygame.draw.line(ecran, NOIR, (startx, starty), (endx, endy))
    #Trace les premiers jetons    
    for x in range(1,NB_COLONNE+1):
        for y in range(1,NB_LIGNE+1):
            centerx, centery = traduirePlateauEnCoordEcran(x, y)
            if plateau[x][y] == JOUEUR or plateau[x][y] == ORDINATEUR:
                if plateau[x][y] == JOUEUR:
                    couleurJeton = BLANC
                else:
                    couleurJeton = NOIR
                pygame.draw.circle(ecran, couleurJeton, (centerx, centery), int(CARRE / 2) - 4)
            
    pygame.display.update()
    
#Affiche en bas de l'écran le nombre de pions deu joueur et de l'ordinateur
def afficher_score(plateau,tour):
    #effacer la ligne des scores
    pygame.draw.rect(ecran,BLANC,(0,HAUTEUR_ECRAN-30,LARGEUR_ECRAN,HAUTEUR_ECRAN)) 
    scores = calculer_score(plateau)
    scoreSurf = FONT.render("Joueur: %s    Ordinateur: %s   Tour: %s" % (str(scores['Joueur']), str(scores['Ordinateur']), tour.title()), True, NOIR)
    ecran.blit(scoreSurf, [10,HAUTEUR_ECRAN - 30])
    pygame.display.update()

#Traduit les coordonnées de la souris au moment du clic en
#numéro de ligne et colonne    
def trouver_caseCliquee(mousex, mousey):
    # Return a tuple of two integers of the board space coordinates where
    # the mouse was clicked. (Or returns None not in any space.)
    for x in range(NB_COLONNE):
        for y in range(NB_LIGNE):
            if mousex > x * CARRE + MARGEX+CADRE/2 and \
               mousex < (x + 1) * CARRE + MARGEX+CADRE/2 and \
               mousey > y * CARRE + MARGEY+CADRE/2 and \
               mousey < (y + 1) * CARRE + MARGEY+CADRE/2:
               return (x+1, y+1)
    return None

#Dessine et change uniquement les jetons à retourner
def retourner_jetons(jetonsAretourner, jetonJoueur):
    if jetonJoueur == JOUEUR:
        couleurJetonsAretourner = BLANC
    elif jetonJoueur == ORDINATEUR:
        couleurJetonsAretourner = NOIR
    else:
        couleurJetonsAretourner = BRUN
        
    for i in range(len(jetonsAretourner)):
        jeton = jetonsAretourner[i]
        ligne = jeton[0]
        colonne = jeton[1]        
        jetonAretournerX, jetonAretournerY = traduirePlateauEnCoordEcran(ligne, colonne)
        pygame.draw.circle(ecran, couleurJetonsAretourner, (jetonAretournerX, jetonAretournerY), int(CARRE / 2) - 4)
    
    pygame.display.update()
    #le jeton brun est visible pendant 3s, pour permettre
    #de bien voir quel est le coup de l'ordinateur
    if jetonJoueur == VISU:
        pygame.time.wait(3000)

#--------------Console -------------------------------------    
#Renvoie False si le choix n'est pas valide
#Si le choix est valide, renvoie une liste des cases à retourner, si le joueur 
#joue cette case
def coup_valide(plateau, joueur, ligne, colonne):
    #Nous vérifions que les coordonnées fournies, correspondent bien à une case
    #du plateau
    if plateau[ligne][colonne] != 0 or not estSurPlateau(ligne, colonne):
        return False
    #Quelle est le chiffre du joueur et celui de son adversaire 
    if joueur == 1:
        adversaire = 2
    else:
        adversaire = 1
    #liste des cases à retourner si on joue cette case
    jetonsARetourner = []
    for xdirection, ydirection in [[0, 1], [1, 1], [1, 0], [1, -1], [0, -1], [-1, -1], [-1, 0], [-1, 1]]:
        #on se place sur la case choisie
        x, y = ligne, colonne
        #on passe à la case adjacente dans une direction donnée
        x += xdirection 
        y += ydirection 
        #on continue dans cette direction tant que l'on est toujours sur
        #le plateau, et que la case sur laquelle on est, contient un pion de
        #l'adversaire.
        while estSurPlateau(x, y) and plateau[x][y] == adversaire:
            #on passe à la case adjacente toujours dans la même direction
            x += xdirection
            y += ydirection
            #Si la case atteinte est sur le plateau et si elle contient un
            #jeton du joueur:
            if estSurPlateau(x, y) and plateau[x][y] == joueur:
                #on repart en sens inverse en notant dans la liste
                #des jetons à retourner les coordonnées [ligne, colonne] des
                #cases dont il faudra modifier le contenu.
                while True:
                    x -= xdirection
                    y -= ydirection
                    if x == ligne and y == colonne:
                        jetonsARetourner.append([x, y])
                        break
                    jetonsARetourner.append([x, y])
    #S'il n'y a aucun jeton à retourner, alors le choix n'est pas valide,
    #sinon on renvoie la liste               
    if len(jetonsARetourner) == 0: 
        return False
    return jetonsARetourner

#Renvoie True si les coordonnées fournies correspondent bien à une case
#du plateau.
def estSurPlateau(x, y):
    return x >= 1 and x <= NB_LIGNE and y >= 1 and y <= NB_COLONNE

#renvoie une liste des coordonnées [ligne, colonne] de toutes les cases
#que le joueur peut jouer.
def trouverCoupsValides(plateau, joueur):
    coupValides = []
    for x in range(1,NB_LIGNE+1):
        for y in range(1,NB_COLONNE+1):
            if coup_valide(plateau, joueur, x, y) != False:
                coupValides.append([x, y])
    return coupValides

#Recherche tous les jetons à retourner si l'on joue la case dont les 
#coordonnées sont fournies et modifie les valeurs sur le plateau.
def jouer_Coup(plateau, joueur, ligne, colonne, jetons):
    #y a t'il des jetons à retourner?
    if jetons == False:
        return False
    #Mettre le chiffre du joueur dans toutes les cases "jeton à retourner"
    plateau[ligne][colonne] = joueur
    for x, y in jetons:
        plateau[x][y] = joueur
    return True

#comptabilise le nombre de pions de chaque joueur et renvoie un dictionnaire
#dont les clés sont 'joueur' et 'ordinateur'
def calculer_score(plateau):
    joueur_score = 0
    ordinateur_score = 0
    for x in range(1,NB_LIGNE+1):
        for y in range(1,NB_COLONNE+1):
            if plateau[x][y] == JOUEUR:
                joueur_score += 1
            if plateau[x][y] == ORDINATEUR:
                ordinateur_score += 1
    return {'Joueur':joueur_score, 'Ordinateur':ordinateur_score}  
#--------------------------IA ordinateur-------------------------------
#Recherche la case à jouer: 
#Pendant les 6 premiers coups, on joue la case qui retourne le moins de pions
#sinon, on revient à IA1
def coup_OrdinateurIA3(plateau, ordinateur):       
    if nb_tour <3:
        return coup_OrdinateurIA4(plateau, ordinateur)

    return coup_OrdinateurIA1(plateau, ordinateur)

#Recherche la case à jouer: si un coin est possible, on le prend
#sinon, on recherche la case qui retourne le plus de pions adverses
def coup_OrdinateurIA1(plateau, ordinateur):
    coupPossibles = trouverCoupsValides(plateau, ordinateur)
    random.shuffle(coupPossibles) 
    # Prendre un coin si possible
    for x, y in coupPossibles:
        if (x == 1 or x == NB_LIGNE) and (y == 1 or y == NB_COLONNE):
            return [x, y]
    #cherche à retourner le plus de pions possibles
    meilleurScore = -1
    for x, y in coupPossibles:
        plateau2 = np.copy(plateau)
        jouer_CoupIA(plateau2, ordinateur, x, y)
        scores = calculer_score(plateau2)
        if ordinateur == 1 :
            scoreOrdi = scores['Joueur']
        else:
            scoreOrdi = scores['Ordinateur']
        if scoreOrdi > meilleurScore:
            meilleurCoup = [x, y]
            meilleurScore = scoreOrdi
    return meilleurCoup

#Recherche la case à jouer: 
#on recherche la case qui retourne le moins de pions adverses
def coup_OrdinateurIA4(plateau, ordinateur):
    coupPossibles = trouverCoupsValides(plateau, ordinateur)
    random.shuffle(coupPossibles) 
    #Cherche à retourner le moins de pions possible
    mauvaisScore = 64
    for x, y in coupPossibles:
        plateau2 = np.copy(plateau)
        jouer_CoupIA(plateau2, ordinateur, x, y)
        scores = calculer_score(plateau2)
        if ordinateur == 1 :
            scoreOrdi = scores['Joueur']
        else:
            scoreOrdi = scores['Ordinateur']
        if scoreOrdi < mauvaisScore:
            mauvaisCoup = [x, y]
            mauvaisScore = scoreOrdi
    return mauvaisCoup

#Recherche tous les jetons à retourner si l'on joue la case dont les 
#coordonnées sont fournies et modifie les valeurs sur le plateau.
def jouer_CoupIA(plateau, joueur, ligne, colonne):
    jetonsARetourner = coup_valide(plateau, joueur, ligne, colonne)
    #y a t'il des jetons à retourner?
    if jetonsARetourner == False:
        return False
    #Mettre le chiffre du joueur dans toutes les cases "jeton à retourner"
    plateau[ligne][colonne] = joueur
    for x, y in jetonsARetourner:
        plateau[x][y] = joueur
    return True

#---------------Démarrage - initialisation ------------------------------
#crée une matrice NB_LIGNE, NB_COLONNE remplie de zeros
#place les premiers jetons sur ce plateau
def creer_plateau():
    plateau = np.zeros((NB_LIGNE+1,NB_COLONNE+1),dtype=int)
    #on numérote les cases de la ligne 0 et colonne 0
    for i in range (1,NB_LIGNE+1):
        plateau[i][0] = i
    for i in range (1,NB_COLONNE+1):
        plateau[0][i] = i
    #Placement des premiers pions    
    plateau[4][4] = 2
    plateau[4][5] = 1
    plateau[5][4] = 1
    plateau[5][5] = 2
    return plateau

#Initialisation du jeu    
def init():
    global plateau, game_over, tour, strTour, nb_tour
    plateau = creer_plateau()
    game_over = False
    dessiner_plateau(plateau)
    i =random.randint(1,2)
    if i==1:
        tour=JOUEUR
        strTour = "Joueur"
    else:
        tour=ORDINATEUR
        strTour = "Ordinateur"
    nb_tour = 0
    pygame.draw.rect(ecran,BLANC,(0,0,300,30))                 
    pygame.display.update()
    afficher_score(plateau,strTour)
# =============================================================================
# Partie principale
# =============================================================================
pygame.init()
MAINCLOCK = pygame.time.Clock()
ecran = pygame.display.set_mode((LARGEUR_ECRAN, HAUTEUR_ECRAN))
pygame.display.set_caption('OTHELLO')
ecran.fill(BLANC)
pygame.display.flip()
FONT = pygame.font.SysFont('comicsansms',18)
init()

fin = False       
#-----------------------------------------------------------------------
while not fin:
    #Boucle de jeu
    while not game_over:                          
         #y a t'il encore des coups possibles
        coupsPossiblesJoueur=trouverCoupsValides(plateau,JOUEUR)
        coupsPossiblesOrdinateur=trouverCoupsValides(plateau,ORDINATEUR)
        if coupsPossiblesJoueur==[] and coupsPossiblesOrdinateur == []:
            game_over = True
            fin_partie = True
            break
        
        for event in pygame.event.get():
            if event.type==pygame.QUIT:
                game_over = True
                pygame.quit()  
                
            if event.type == pygame.MOUSEBUTTONUP and tour==JOUEUR:
                mousex, mousey = event.pos
                #détecter si clic sur bouton nouveau jeu
                if zone_NouveauJeu.collidepoint( (mousex, mousey) ):
                    #réinitialisation
                    init()
                    continue
                #clic du joueur dans une case    
                choix = trouver_caseCliquee(mousex, mousey)
                coupsPossiblesJoueur=trouverCoupsValides(plateau,JOUEUR)
                if coupsPossiblesJoueur!=[]:
                    #jouer le coup du joueur
                    jetonsAretourner = coup_valide(plateau, JOUEUR, choix[0], choix[1])
                    if  jetonsAretourner :
                        jouer_Coup(plateau, JOUEUR, choix[0], choix[1], jetonsAretourner) 
                        retourner_jetons(jetonsAretourner, JOUEUR)
                        tour = ORDINATEUR
                        strTour = "Ordinateur"
                        afficher_score(plateau,strTour)
                    else:
                        #le joueur n'a pas cliqué sur une case valide
                        #on ne tient pas compte de son clic
                        pass
                else:
                    tour = ORDINATEUR
                    strTour = "Ordinateur"
                    afficher_score(plateau,strTour)
                
            if tour==ORDINATEUR:
            #l'ordinateur a t'il encore des cases à jouer
                coupsPossiblesOrdinateur=trouverCoupsValides(plateau,ORDINATEUR)
                if coupsPossiblesOrdinateur!=[]:
                    choix = coup_OrdinateurIA3(plateau, ORDINATEUR)
                    #on affiche juste le coup joué avec un jeton marron
                    jetonsAretourner=[choix]
                    retourner_jetons(jetonsAretourner, VISU)
                    #On joue le coup et on retourne tous les jetons
                    jetonsAretourner = coup_valide(plateau, ORDINATEUR, choix[0], choix[1])
                    jouer_Coup(plateau, ORDINATEUR, choix[0], choix[1], jetonsAretourner)                     
                    retourner_jetons(jetonsAretourner, ORDINATEUR)
                    nb_tour+=1 
                tour = JOUEUR
                strTour = "Joueur"
                afficher_score(plateau,strTour)
                
    #fin de partie : affichage du gagnant         
    if fin_partie:
        scores = calculer_score(plateau)
        if scores['Joueur'] > scores['Ordinateur']:
            scoreTexte = FONT.render("Vous avez gagné", True, NOIR)            
        elif scores['Joueur'] < scores['Ordinateur']:
            scoreTexte = FONT.render("L'ordinateur gagne", True, NOIR)
        else:
            scoreTexte = FONT.render("Partie nulle", True, NOIR)
        ecran.blit(scoreTexte, [100,5])
        pygame.display.update()
        fin_partie = False
        
    for event in pygame.event.get():
        if event.type==pygame.QUIT:
            game_over = True
            pygame.quit()  
            
        if event.type == pygame.MOUSEBUTTONUP:
            mousex, mousey = event.pos       
            #détecter si clic sur bouton nouveau jeu
            if zone_NouveauJeu.collidepoint( (mousex, mousey) ):
                init()                 
        
pygame.quit()